import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Lab #4 modified to show type-safe object input with exceptions
 * CS 2334, Section 010
 * 19 November 2008
 * <P>
 * This class models a word.
 * </P>
 * @author Dean Hougen
 * @version 1.0
 */
public class SafeWord implements Serializable {
	private static final long serialVersionUID = -2668044860460453424L;
	/** The individual word */
	private String word;
	/** The language of origin for the word */
	private String language;

	/**
	 * This is the default constructor for the class.
	 */   
	public SafeWord() 
	{
		word = null;
		language = null;
	}

	/**
	 * This is the constructor for the class. 
	 * It instantiates the class with user supplied values.
	 * <P>
	 * @param             city    The name of the word.
	 * @param             year    The language or origin for the word.
	 */   
	public SafeWord(String word, String language) 
	{
		this.word = word;
		this.language = language;
	}

	/**
	 * This method returns the attributes of this class as 
	 * a single string.
	 * </P>
	 * @return            String representing the contents of this object.
	 */
	public String toString()
	{
		return word + ", " + language;
	}

	/**
	 * This method writes the word to the file of the given name using object IO.
	 * <p>
	 * @param 			fileName	The name of the file to write to.
	 * @param 			word 	 	The word to write to the file.
	 * @throws IOException 
	 */
	public static void writeWord(String fileName, SafeWord word) throws IOException {
		FileOutputStream fos = new FileOutputStream(fileName);
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(word);
		oos.close();
	}

	/**
	 * This method reads in a word from the file of the given name using object IO.
	 * <p>
	 * @param 			fileName 	The name of the file to read from.
	 * @return 						The word read in from the file.
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 */
	public static SafeWord readWord(String fileName) throws IOException, ClassNotFoundException {
		FileInputStream fis = new FileInputStream(fileName);
		ObjectInputStream ois = new ObjectInputStream(fis);
		try {
			SafeWord word = (SafeWord) ois.readObject();
			return word;
		}
		catch (ClassCastException exception){
			System.out.println ("Hey, that ain't no Word!  Check it out: " + exception);
			return null;
		}
		finally {
			ois.close();
		}
	}

	/**
	 * This method writes a whole list of words with one magical line of object IO.
	 * <p>
	 * @param 			fileName	The name of the file to write to.
	 * @param 			words		The words to write to the file.
	 * @throws IOException 
	 */
	public static void writeDictionary(String fileName, List<SafeWord> words) throws IOException {
		FileOutputStream fos = new FileOutputStream(fileName);
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(words);
		oos.close();
	}

	/**
	 * This method reads in a list of objects from a file with one magical line of object IO.
	 * <p>
	 * @param 			fileName 	The name of the file to read from.
	 * @return						The list of words read in from the file.
	 * @throws IOException
	 * @throws ClassNotFoundException 
	 */
	public static List<SafeWord> readDictionary(String fileName) throws IOException, ClassNotFoundException {
		FileInputStream fis = new FileInputStream(fileName);
		ObjectInputStream ois = new ObjectInputStream(fis);
		try {
			List<?> words = (List<?>) ois.readObject();
			return copyWordsFromList(words);
		}
		catch (ClassCastException exception){
			System.out.println ("Hey, that ain't no List!  Check it out: " + exception);
			return null;
		}
		finally {
			ois.close();
		}
	}

	/**
	 * This method converts a list of objects of unknown type(s) to a list of Word objects.
	 * Type safety is achieved by using the add() method of the Java Collections Framework.
	 * Note that add() will only add the object to the list if it is of an appropriate type
	 * for the collection.  So, if any objects of other types are in the List<?>, they will
	 * not be copied to the List<Word> collection.  Rather, a ClassCastException will be thrown.
	 * Note that you could make this all more generic by changing it to something like
	 * CopyTypeFromList and then using a generic type T rather than Word as your type.
	 * @param 			wildList 	The list of unknown type.
	 * @return						The list of words.
	 */
	public static List<SafeWord> copyWordsFromList (List<?> wildList) {
		List<SafeWord> wordList = new ArrayList<SafeWord>();
		for (Object obj: wildList){
			try {
				SafeWord word = (SafeWord) obj;
				wordList.add(word);
			}
			catch (ClassCastException exception){
				System.out.println ("Hey, that ain't no Word!  Check it out: " + exception);
			}
		}
		return wordList;
	}
}