import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Lab #4 completed
 * CS 2334, Section 010
 * 19 November 2008
 * <P>
 * This class models a word.
 * </P>
 * @author Dean Hougen
 * @version 1.0
 */
public class Word implements Serializable {
	private static final long serialVersionUID = -2668044860460453424L;
	/** The individual word */
	private String word;
	/** The language of origin for the word */
	private String language;

	/**
	 * This is the default constructor for the class.
	 */   
	public Word() 
	{
		word = null;
		language = null;
	}

	/**
	 * This is the constructor for the class. 
	 * It instantiates the class with user supplied values.
	 * <P>
	 * @param             city    The name of the word.
	 * @param             year    The language or origin for the word.
	 */   
	public Word(String word, String language) 
	{
		this.word = word;
		this.language = language;
	}

	/**
	 * This method returns the attributes of this class as 
	 * a single string.
	 * </P>
	 * @return            String representing the contents of this object.
	 */
	public String toString()
	{
		return word + ", " + language;
	}

	/**
	 * This method writes the word to the file of the given name using object IO.
	 * <p>
	 * @param 			fileName	The name of the file to write to.
	 * @param 			word 	 	The word to write to the file.
	 * @throws IOException 
	 */
	public static void writeWord(String fileName, Word word) throws IOException {
		FileOutputStream fos = new FileOutputStream(fileName);
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(word);
		oos.close();
	}

	/**
	 * This method reads in a word from the file of the given name using object IO.
	 * <p>
	 * @param 			fileName 	The name of the file to read from.
	 * @return 						The word read in from the file.
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 */
	public static Word readWord(String fileName) throws IOException, ClassNotFoundException {
		FileInputStream fis = new FileInputStream(fileName);
		ObjectInputStream ois = new ObjectInputStream(fis);
		Word word = (Word) ois.readObject();
		ois.close();
		return word;
	}

	/**
	 * This method writes a whole list of words with one magical line of object IO.
	 * <p>
	 * @param 			fileName	The name of the file to write to.
	 * @param 			words		The words to write to the file.
	 * @throws IOException 
	 */
	public static void writeDictionary(String fileName, List<Word> words) throws IOException {
		FileOutputStream fos = new FileOutputStream(fileName);
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(words);
		oos.close();
	}

	/**
	 * This method reads in a list of objects from a file with one magical line of object IO.
	 * <p>
	 * @param 			fileName 	The name of the file to read from.
	 * @return						The list of words read in from the file.
	 * @throws IOException
	 * @throws ClassNotFoundException 
	 */
	public static List<Word> readDictionary(String fileName) throws IOException, ClassNotFoundException {
		FileInputStream fis = new FileInputStream(fileName);
		ObjectInputStream ois = new ObjectInputStream(fis);
		List<Word> words = (List<Word>) ois.readObject();  // Note the warning here!  Type Safety: Unchecked cast from Object to List<Word>
		ois.close();
		return words;
	}
}